#!/usr/bin/perl -w
################################################################################
# fan_backup.pl : backup FAN server datas to on directory.
# Copyright (C) 2012 FAN Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
################################################################################
# Description : backup FAN server datas to on directory.
################################################################################
# Version : 2.0
################################################################################
# Author : Cedric TEMPLE <cedrictemple@cedrictemple.info>
#          Olivier LI KIANG CHEONG <olivier@gezen.fr>    
################################################################################

use strict;
use Getopt::Long;
use vars qw($mysql_user $mysql_passwd $mysql_host $mysql_database_oreon $mysql_database_ods $mysql_database_ndo);
use vars qw($opt_h $opt_d $opt_c $opt_p $opt_s);

if (-e '/etc/centreon/conf.pm') {
    require '/etc/centreon/conf.pm';
} else{
    $mysql_user = "root";
    $mysql_passwd = "";
    $mysql_host = "localhost";
    $mysql_database_oreon = "centreon2";
    $mysql_database_ods = "centreon2_storage";
}

my $progname = $0;
my $current_debug_level = 3;
my $backup_dir = '/var/backup';
my $mysql_database_ndo = 'nagios';

# backup database $1 to directory $2. Use user $3 and password $4 to connect
# password must be set to '' to something
sub backup_database($$$$);

# backup directory source $1 to directory destination $2.
sub backup_directory($$);

# log message $1 in a log file if level $2 >= current debug level
sub debug($$);

# print usage
sub usage();

# print help
sub help();

# main
Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"             => \$opt_h,
     "d"   => \$opt_d, "fan-database"     => \$opt_d,
     "c"   => \$opt_c, "fan-central"      => \$opt_c,
     "p"   => \$opt_p, "fan-poller"       => \$opt_p,
     "s"   => \$opt_s, "fan-standalone"   => \$opt_s
);

my $date = `date +%Y%m%d_%H%M%S`;
$date =~ s/\s+//g;
$backup_dir .= '/'.$date;

if ($opt_h) {
    usage();
    help();
} elsif ($opt_d) {
    backup_database($mysql_database_oreon, $backup_dir.'/databases/'.$mysql_database_oreon, $mysql_user, $mysql_passwd);
    backup_database($mysql_database_ods, $backup_dir.'/databases/'.$mysql_database_ods, $mysql_user, $mysql_passwd);
    backup_database($mysql_database_ndo, $backup_dir.'/databases/'.$mysql_database_ndo, $mysql_user, $mysql_passwd);
} elsif ($opt_c) {
    backup_directory( '/usr/share/nagios/images/logos/', $backup_dir);
    backup_directory( '/usr/lib/nagios/plugins/', $backup_dir);
    backup_directory( '/var/www/dokuwiki/data/pages/', $backup_dir);
    backup_directory( '/etc/centreon/', $backup_dir);
    backup_directory( '/etc/nagvis/', $backup_dir);
} elsif ($opt_p) {
    backup_directory( '/usr/share/nagios/images/logos/', $backup_dir);
    backup_directory( '/usr/lib/nagios/plugins/', $backup_dir);
    backup_directory( '/etc/centreon/', $backup_dir);
} elsif ($opt_s) {
    backup_database($mysql_database_oreon, $backup_dir.'/databases/'.$mysql_database_oreon, $mysql_user, $mysql_passwd);
    backup_database($mysql_database_ods, $backup_dir.'/databases/'.$mysql_database_ods, $mysql_user, $mysql_passwd);
    backup_database($mysql_database_ndo, $backup_dir.'/databases/'.$mysql_database_ndo, $mysql_user, $mysql_passwd);
    backup_directory( '/usr/share/nagios/images/logos/', $backup_dir);
    backup_directory( '/usr/lib/nagios/plugins/', $backup_dir);
    backup_directory( '/var/www/dokuwiki/data/pages/', $backup_dir);
    backup_directory( '/etc/centreon/', $backup_dir);
    backup_directory( '/etc/nagvis/', $backup_dir);
} else{
    usage();
}

# backup database $1 to directory $2. Use user $3 and password $4 to connect
# password must be set to '' to something
sub backup_database($$$$){
    my ($db, $dir, $user, $pass) = ($_[0], $_[1], $_[2], $_[3]);
    my $error_message;
    my $return_code;

    print "Backup database $db\n";
    debug( 'backup database '.$_[0] . ' to directory '. $_[1].' using user '.$_[2].' and password '.$_[3], 2);

    if ( ! -d $dir ) {
	$error_message = `mkdir -p $dir 2>&1`;
	$return_code = $?;
	if ( $return_code != 0 ) {
	    debug( 'ERROR! Could NOT create directory '.$dir.' Error : '.$error_message, 0);
	}
    }
    else {
	debug( "Directory $dir exists. I do not have to create it!", 2 );
    }
    
    my $mysqldump_args = "--opt $db --user=$user --host=$mysql_host";
    
    if ( defined($pass) && ($pass ne "")) {
	$mysqldump_args .= " --password=$pass > $dir/$db.sql 2> /dev/null";
    }
    else {
	$mysqldump_args .= " > $dir/$db.sql 2> /dev/null";
    }
    debug( 'launch mysqldump '.$mysqldump_args, 2);
    `mysqldump $mysqldump_args`;
    $return_code = $?;
    if ( $return_code != 0 ) {
	debug( 'ERROR! Could NOT backup database '.$db, 0);
    }

    
    debug( "gzip -9 $dir/$db.sql", 2);
    $error_message = `gzip -9 $dir/${db}.sql 2>&1`;
    $return_code = $?;
    if ( $return_code != 0 ) {
        debug( 'ERROR! Could NOT compress file '.$dir."/".$db.".sql".' Error message :'.$error_message, 0);
    } else{
        print "    File $dir/$db.sql.gz created\n";
    }

}

# log message $1 in a log file if level $2 <= current debug level
sub debug($$){
    if ( $_[1] <= $current_debug_level ) {
	open( DEBUG_FILE, ">> /var/log/fan_backup.log") or die ("Could not open log file /var/log/fan_backup");
	printf DEBUG_FILE "DEBUG %d : %s\n", $_[1], $_[0];
	close ( DEBUG_FILE ) ;
    }

}

# backup directory source $1 to directory destination $2.
sub backup_directory($$){
    my ($source, $destination) = ($_[0], $_[1]);
    my $error_message;
    my $return_code;

    print('Backup directory '.$_[0]."\n");
    debug( 'backup directory source '.$_[0] . ' to directory destination '. $_[1], 2);

    if ( ! -d $destination ) {
	$error_message = `mkdir -p $destination 2>&1`;
	$return_code = $?;
	if ( $return_code != 0 ) {
	    debug( 'ERROR! Could NOT create directory '.$destination.' Error : '.$error_message, 0);
	}
    }
    else {
	debug( "Directory $destination exists. I do not have to create it!", 2 );
    }
    
    debug( "cp -a $source $destination", 2);
    $error_message = `cp -a $source $destination 2>&1`;
    $return_code = $?;
    if ( $return_code != 0 ) {
        debug( 'ERROR! Could NOT copy directory'. $source.' to directory '. $destination.'. Error message :'.$error_message, 0);
    } else{
        print( '    Backup to '. $_[1]."\n");
    }
    
}

sub usage() {
    print "Usage : $progname [-h|--help] | [-d|--fan-database] | [-c|--fan-central] | [-p|--fan-poller] | [-s|--fan-standalone]\n";
}

sub help() {
    print "   -d|--fan-database : Used on FAN Database, backup mysql databases (centreon2, centreon2_storage, nagios)\n";
    print "   -c|--fan-central : Used on FAN Central, backup files centreon, nagios plugins, nagvis, nagios logos, dokuwiki\n";
    print "   -p|--fan-poller : Used on FAN Poller, backup files nagios plugins, nagios logos\n";
    print "   -s|--fan-standalone : Used on FAN Standalone, backup all files and databases mysql\n"
}
